%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lists.h"
  
#define TAILLEBUFF 1024

  extern void yyerror(const char *str);
  extern FILE* yyin;
  extern int yylineno;

  List declarations=NULL;

  /* Compteur d'instructions */
  int instrno=0;
  /* Compteur d'etiquetes */
  int label_no=0; 
  
  void build(char *);
  void branch(int);
  void load(int);
  void add(void);
  void ret(void);
  void make_a_label(int);

%}


%union {
  char* str;
  int label;
}

%left PLUS
%token LPAR RPAR
%token IF  ELSE
%token <label> THEN
%token LET EQ IN
%token <str> ID 
%token <str> CONST

%start S


%%
 /*Begin solution*/

S:
b_expression 

b_expression:
e_expression {ret();}
| LPAR b_expression RPAR 
| LET ID EQ e_expression 
{declarations=list_add($2,declarations);}
IN b_expression 
| IF e_expression THEN {
  $3=label_no++;
  branch($3);
}
b_expression ELSE {make_a_label($3);} b_expression

e_expression:
ID {load(list_index($1,declarations));}
| CONST {build($1);}
| e_expression PLUS  e_expression {add();}
| LPAR e_expression RPAR 

%%

void yyerror(char const *str)
{
  fprintf(stderr,"Ligne %d : %s\n", yylineno, str); 
  exit(EXIT_FAILURE);
}


int 
main(int argc, char* argv[])
{
  if ( argc > 1 ) {
    if((yyin = fopen( argv[1], "r" ))==NULL)
      exit(EXIT_FAILURE);
    
    yyparse();
    fclose(yyin);
  }
  else exit(EXIT_FAILURE);

  exit(EXIT_SUCCESS);
}


void
build(char * s){
  printf("\t%d : \tbuild \t%s\n",++instrno,s);
}

void
branch(int label){
  printf("\t%d : \tbranch \tl%d\n",++instrno,label);
}

void
make_a_label(int i){
   printf("l%d:\n",i);
}

void
load(int v){
  printf("\t%d : \tload \t%d\n",++instrno,v);
}

void
add(void){
  printf("\t%d : \tadd\n",++instrno);
}

void
ret(void){
  printf("\t%d : \treturn\n",++instrno);
}

